﻿//---------------------------------------------------------------------------------
// A flash vector class
// #Author unknown
//---------------------------------------------------------------------------------
class Vector
{
	var x: Number;
	var y: Number;
	
	var lengthDirty: Boolean;
	var ln: Number;
	
	//-- CONSTRUCTOR
	function Vector( x: Number, y: Number )
	{
		this.x = x || 0;
		this.y = y || 0;
		lengthDirty = true;
	}
	
	function length( Void ): Number
	{
		if (lengthDirty){
			ln = Math.sqrt( x * x + y * y );
			lengthDirty = false;
		}
		return ln;
	}
	
	function angle( Void ): Number
	{
		return Math.atan2( y , x );
	}
	
	function dot( v: Vector ): Number
	{
		return x * v.x + y * v.y;
	}
	
	function cross( v: Vector ): Number
	{
		return x * v.y - y * v.x;
	}
		
	function angleBetween( v: Vector ): Number
	{
		return Math.acos ( dot( v ) / ( this.length() * v.length() ) );
	}
	
	function null( Void ): Vector
	{
		x = y = 0;
		return this;
	}
	
	function reflect( normal: Vector ): Vector
	{
		var dp: Number = 2 * dot( normal );
		
		x -= normal.x * dp;
		y -= normal.y * dp;
		
		lengthDirty = true;
		
		return this;
	}
	
	function getReflect( normal: Vector ): Vector
	{
		var dp: Number = 2 * dot( normal );
		
		return new Vector( x - normal.x * dp, y - normal.y * dp );
	}
	
	function negate( Void ): Vector
	{
		x = -x;
		y = -y;
		
		return this;
	}
	
	function getNegate( Void ): Vector
	{
		return new Vector( -x , -y );
	}
	
	function orth( Void ): Vector
	{
		var tx: Number = -y;
		
		y = x;
		x = tx;
		
		return this;
	}
	
	function getOrth( Void ): Vector
	{
		return new Vector ( -y , x );
	}
	
	function normalize( Void ): Vector
	{
		var l: Number = this.length();
		
		x /= l;
		y /= l;
		
		ln = 1;
		lengthDirty = false;
		
		return this;
	}
	
	function getNormalize( Void ): Vector
	{
		var l: Number = this.length();
		
		return new Vector( x / l, y / l );
	}
	
	function normal( Void ): Vector
	{
		var l: Number = this.length();
		var tx: Number = -y / l;

		y = x / ln;
		x = tx;
		
		ln = 1;
		lengthDirty = false;
		
		return this;
	}
	
	function getNormal( Void ): Vector
	{
		var l: Number = this.length();
		
		return new Vector ( -y / l , x / l );
	}
	
	function rotateBy( angle: Number ): Vector
	{
		var ca: Number = Math.cos ( angle );
		var sa: Number = Math.sin ( angle );
		var rx: Number = x * ca - y * sa;
		y = x * sa + y * ca;
		x = rx;
		
		return this;
	}
	
	function rotateByCosSin( ca: Number, sa: Number ): Vector
	{
		var rx: Number = x * ca - y * sa;
		y = x * sa + y * ca;
		x = rx;
		
		return this;
	}
	
	function getRotateBy( angle: Number ): Vector
	{
		var ca: Number = Math.cos ( angle );
		var sa: Number = Math.sin ( angle );
		var rx: Number = x * ca - y * sa;
		
		return new Vector( rx , x * sa + y * ca );
	}
	
	function getRotateByCosSin( ca: Number, sa: Number ): Vector
	{
		var rx: Number = x * ca - y * sa;
		
		return new Vector( rx , x * sa + y * ca );
	}
	
	function rotateTo( angle: Number ): Vector
	{
		var l: Number = this.length();
		x = Math.cos( angle ) * l;
		y = Math.sin( angle ) * l;
		
		return this;
	}
	
	function getRotateTo( angle: Number ): Vector
	{
		var l: Number = this.length();
		
		return new Vector( Math.cos( angle ) * l, Math.sin( angle ) * l );
	}
	
	function newLength( len: Number ): Vector
	{
		var l: Number = this.length();
		
		x /= l / len;
		y /= l / len;
		
		ln = Math.abs( len );
		lengthDirty = false;
		
		return this;
	}
	
	function getNewLength( len: Number ): Vector
	{
		var l: Number = this.length();
		
		return new Vector( x / l * len, y / l * len );
	}
	
	function plus( v: Vector ): Vector
	{
		x += v.x;
		y += v.y;
		
		lengthDirty = true;
		
		return this;
	}
	
	function getPlus( v: Vector ): Vector
	{
		return new Vector( x + v.x, y + v.y );
	}
	
	function minus( v: Vector ): Vector
	{
		x -= v.x;
		y -= v.y;
		
		lengthDirty = true;
		
		return this;
	}
	
	function getMinus( v: Vector ): Vector
	{
		return new Vector( x - v.x, y - v.y );
	}
	
	function multiply( f: Number ): Vector
	{
		x *= f;
		y *= f;
		
		lengthDirty = true;
		
		return this;
	}
	
	function getMultiply( f: Number ): Vector
	{
		return new Vector( x * f, y * f );
	}
	
	function divide( d: Number ): Vector
	{
		x /= d;
		y /= d;
		
		lengthDirty = true;
		
		return this;
	}
	
	function getDivide( d: Number ): Vector
	{
		return new Vector( x / d, y / d );
	}
	
	function getProject(v:Vector) : Vector
	{
		var vn:Vector = v.getNormalize();
		var dp:Number = vn.dot(this);
		return new Vector(dp*vn.x, dp*vn.y);
	}
	
	function getClone( Void ): Vector
	{
		return new Vector( x , y );
	}
	
	function toString( Void ): String
	{
		var rx: Number = Math.round ( x * 1000 ) / 1000;
		var ry: Number = Math.round ( y * 1000 ) / 1000;
		
		return '[Vector x: ' + rx + ' y: ' + ry + ' length: ' + this.length() + ']';
	}
}